@@ -14,6 +14,8 @@ from ast import Attribute
 from ast import FunctionDef
 from ast import NodeVisitor
 
+from .utils import YieldSearch
+
 
 ClassDef = namedtuple('ClassDef', 'name')
 FunctionDef = namedtuple('FunctionDef', 'name')
@@ -78,16 +80,19 @@ class Pythonium(NodeVisitor):
 
     def visit_ExceptHandler(self, node):
         # 'type', 'name', 'body'
-        self.writer.write('if (pythonium_is_exception(__exception__, {}) {{'.format(self.visit(node.type)))
-        self.writer.push()
-        self.writer.write('var {} = __exception__'.format(node.name))
+        if node.type:
+            self.writer.write('if (pythonium_is_exception(__exception__, {}) {{'.format(self.visit(node.type)))
+            self.writer.push()
+        if node.name:
+            self.writer.write('var {} = __exception__'.format(node.name))
         list(map(self.visit, node.body))
-        self.writer.pull()
-        self.writer.write('}')
+        if node.type:
+            self.writer.pull()
+            self.writer.write('}')
 
     def visit_Yield(self, node):
-        self.writer.write('yield {};'.format(self.visit(node.value)))
-
+        return 'yield {}'.format(self.visit(node.value))
+        
     def visit_In(self, node):
         return ' in '
 
@@ -156,16 +161,26 @@ class Pythonium(NodeVisitor):
             name = '__{}_{}'.format(self._def_stack[-2].name, node.name)
         else:
             name = node.name
-        self.writer.write('var {} = function({}) {{'.format(name, __args))
+        # handle yield
+        has_yield = False
+        for child in node.body:
+            searcher = YieldSearch()
+            searcher.visit(child)
+            if getattr(searcher, 'has_yield', False):
+                has_yield = True
+                break
+        if has_yield:
+            self.writer.write('var {} = function*({}) {{'.format(name, __args))
+        else:
+            self.writer.write('var {} = function({}) {{'.format(name, __args))
         self.writer.push()
         if not varkwargs:
             varkwargs = '__kwargs'
 
         # unpack arguments
-        # unpack arguments
-        if varargs or varkwargs != '__kwargs':
+        if varargs or varkwargs != '__kwargs' or kwargs:
             self.writer.write('var __args = Array.prototype.slice.call(arguments);')
-        if varkwargs != '__kwargs':
+        if varkwargs != '__kwargs' or kwargs:
             self.writer.write('var {} = __args[__args.length - 1];'.format(varkwargs))
         for keyword in kwargs.keys():
             self.writer.write('{} = {} || {}.{} || {};'.format(keyword, keyword, varkwargs, keyword, kwargs[keyword]))
@@ -214,9 +229,9 @@ class Pythonium(NodeVisitor):
         return node.name, name 
 
     def visit_Slice(self, node):
-        start = self.visit(node.lower)
-        end = self.visit(node.upper)
-        step = self.visit(node.step)
+        start = self.visit(node.lower) if node.lower else 'undefined'
+        end = self.visit(node.upper) if node.upper else 'undefined'
+        step = self.visit(node.step) if node.step else 'undefined'
         return 'slice({}, {}, {})'.format(start, step, end)
 
     def visit_Index(self, node):
@@ -342,8 +357,11 @@ class Pythonium(NodeVisitor):
             self.writer.write('if(!{}) {{ continue; }}'.format(' && '.join(map(self.visit, node.ifs))))
 
     def visit_While(self, node):
-        self.writer.write('while({}) {{'.format(node.test))
+        self.writer.write('while({}) {{'.format(self.visit(node.test)))
+        self.writer.push()
         list(map(self.visit, node.body))
+        self.writer.pull()
+        self.writer.write('}')
 
     def visit_AugAssign(self, node):
         target = self.visit(node.target)
@@ -352,8 +370,8 @@ class Pythonium(NodeVisitor):
     def visit_Str(self, node):
         s = node.s.replace('\n', '\\n')
         if '"' in s:
-            return "str('{}')".format(s)
-        return 'str("{}")'.format(s)
+            return "pythonium_call(str, '{}')".format(s)
+        return 'pythonium_call(str, "{}")'.format(s)
 
     def visit_BinOp(self, node):
         left = self.visit(node.left)
@@ -439,9 +457,18 @@ class Pythonium(NodeVisitor):
     def visit_Assign(self, node):
         value = self.visit(node.value)
         if len(node.targets) == 1 and not isinstance(node.targets[0], Tuple):
-            target = self.visit(node.targets[0])
-            self.writer.write('{} = {};'.format(target, value))
-            return
+            target = node.targets[0]
+            if isinstance(target, Attribute):
+                self.writer.write('pythonium_set_attribute({}, "{}", {});'.format(
+                    self.visit(target.value),
+                    target.attr.replace('__DOLLAR__', '$'),
+                    value
+                ))
+                return
+            else:
+                target = self.visit(target)
+                self.writer.write('{} = {};'.format(target, value))
+                return
 
         self.writer.write('var __assignement = {};'.format(value))
         for target in node.targets:
@@ -547,7 +574,10 @@ class Pythonium(NodeVisitor):
         self.writer.write('}')
 
     def visit_Continue(self, node):
-        return 'continue'
+        self.writer.write('continue;')
+
+    def visit_Break(self, node):
+        self.writer.write('break;')
 
     def visit_Lambda(self, node):
         args = ', '.join(map(self.visit, node.args.args))
@@ -556,7 +586,7 @@ class Pythonium(NodeVisitor):
     def visit_ClassDef(self, node):
         # 'name', 'bases', 'keywords', 'starargs', 'kwargs', 'body', 'decorator_lis't
         if len(node.bases) == 0:
-            bases = ['pythonium_object']
+            bases = ['object']
         else:
             bases = map(self.visit, node.bases)
         bases = '[{}]'.format(', '.join(bases))
