@@ -12,6 +12,8 @@ from ast import Global
 from ast import FunctionDef
 from ast import NodeVisitor
 
+from .utils import YieldSearch
+
 
 class Writer:
 
@@ -74,7 +76,7 @@ class Veloce(NodeVisitor):
         list(map(self.visit, node.body))
 
     def visit_Yield(self, node):
-        self.writer.write('yield {};'.format(self.visit(node.value)))
+        return 'yield {}'.format(self.visit(node.value))
 
     def visit_In(self, node):
         return ' in '
@@ -129,21 +131,36 @@ class Veloce(NodeVisitor):
             all_parameters.append(varkwargs)
         all_parameters = set(all_parameters)
 
+        name = node.name
+        # handle yield
+        has_yield = False
+        for child in node.body:
+            searcher = YieldSearch()
+            searcher.visit(child)
+            if getattr(searcher, 'has_yield', False):
+                has_yield = True
+                break
         if self.in_classdef and len(self._function_stack) == 1:
             __args = ', '.join(args[1:])
-            self.writer.write('{}: function({}) {{'.format(node.name, __args))
+            if has_yield:
+                self.writer.write('{}: function*({}) {{'.format(name, __args))
+            else:
+                self.writer.write('{}: function({}) {{'.format(name, __args))
         else:
             __args = ', '.join(args)
-            self.writer.write('var {} = function({}) {{'.format(node.name, __args))
+            if has_yield:
+                self.writer.write('var {} = function*({}) {{'.format(name, __args))
+            else:
+                self.writer.write('var {} = function({}) {{'.format(name, __args))
         self.writer.push()
         if not varkwargs:
             varkwargs = '__kwargs'
 
         # unpack arguments
-        if varargs or varkwargs != '__kwargs':
+        if varargs or varkwargs != '__kwargs' or kwargs:
             self.writer.write('var __args = Array.prototype.slice.call(arguments);')
-        if varkwargs != '__kwargs':
-            self.writer.write('var {} = __args[__args.length - 1];'.format(varkwargs))
+        if varkwargs != '__kwargs' or kwargs:
+            self.writer.write('var {} = __args[__args.length - 1] || {{}};'.format(varkwargs))
         for keyword in kwargs.keys():
             self.writer.write('{} = {} || {}.{} || {};'.format(keyword, keyword, varkwargs, keyword, kwargs[keyword]))
             self.writer.write('delete {}.{};'.format(varkwargs, keyword))
@@ -193,8 +210,17 @@ class Veloce(NodeVisitor):
             self.writer.write('{} = {}({});'.format(node.name, decorator, node.name))
         self._function_stack.pop()
 
+    def visit_Slice(self, node):
+        start = self.visit(node.lower) if node.lower else 'undefined'
+        end = self.visit(node.upper) if node.upper else 'undefined'
+        step = self.visit(node.step) if node.step else 'undefined'
+        return 'slice({}, {}, {})'.format(start, step, end)
+
+    def visit_Index(self, node):
+        return self.visit(node.value)
+
     def visit_Subscript(self, node):
-        return '{}[{}]'.format(self.visit(node.value), self.visit(node.slice.value))
+        return '{}[{}]'.format(self.visit(node.value), self.visit(node.slice))
 
     def visit_arguments(self, node):
         # 'args', 'vararg', 'varargannotation', 'kwonlyargs', 'kwarg', 'kwargannotation', 'defaults', 'kw_defaults'
@@ -512,7 +538,10 @@ class Veloce(NodeVisitor):
         self.writer.write('}')
 
     def visit_Continue(self, node):
-        return 'continue'
+        self.writer.write('continue;')
+
+    def visit_Break(self, node):
+        self.writer.write('break;')
 
     def visit_Lambda(self, node):
         args = ', '.join(map(self.visit, node.args.args))
